@@ -244,8 +244,11 @@ class IMAPFolder(BaseFolder):
 
             # Get the flags and UIDs for these. single-quotes prevent
             # imaplib2 from quoting the sequence.
-            res_type, response = imapobj.fetch("'%s'"%
-                msgsToFetch, '(FLAGS UID INTERNALDATE)')
+            fetch_msg = "'%s'"% msgsToFetch
+            self.ui.debug('imap', "calling imaplib2 fetch command: %s %s"%
+                (fetch_msg, '(FLAGS UID INTERNALDATE)'))
+            res_type, response = imapobj.fetch(
+                fetch_msg, '(FLAGS UID INTERNALDATE)')
             if res_type != 'OK':
                 raise OfflineImapError("FETCHING UIDs in folder [%s]%s failed. "
                     "Server responded '[%s] %s'"% (self.getrepository(), self,
@@ -256,11 +259,11 @@ class IMAPFolder(BaseFolder):
         for messagestr in response:
             # Looks like: '1 (FLAGS (\\Seen Old) UID 4807)' or None if no msg.
             # Discard initial message number.
-            if messagestr == None:
+            if messagestr is None:
                 continue
             messagestr = messagestr.split(' ', 1)[1]
             options = imaputil.flags2hash(messagestr)
-            if not 'UID' in options:
+            if 'UID' not in options:
                 self.ui.warn('No UID in message with options %s'%
                     str(options), minor=1)
             else:
@@ -829,19 +832,22 @@ class IMAPFolder(BaseFolder):
             except imapobj.readonly:
                 self.ui.flagstoreadonly(self, uidlist, flags)
                 return
-            r = imapobj.uid('store',
+            response = imapobj.uid('store',
                 imaputil.uid_sequence(uidlist), operation + 'FLAGS',
                     imaputil.flagsmaildir2imap(flags))
-            assert r[0] == 'OK', 'Error with store: ' + '. '.join(r[1])
-            r = r[1]
+            if response[0] != 'OK':
+                raise OfflineImapError(
+                    'Error with store: %s'% '. '.join(response[1]),
+                    OfflineImapError.ERROR.MESSAGE)
+            response = response[1]
         finally:
             self.imapserver.releaseconnection(imapobj)
         # Some IMAP servers do not always return a result.  Therefore,
         # only update the ones that it talks about, and manually fix
         # the others.
         needupdate = list(uidlist)
-        for result in r:
-            if result == None:
+        for result in response:
+            if result is None:
                 # Compensate for servers that don't return anything from
                 # STORE.
                 continue
